<?php

namespace Drupal\deepgram;

use Drupal\deepgram\Form\DeepgramConfigForm;
use Drupal\Core\Config\ConfigFactory;
use Drupal\file\Entity\File;
use Drupal\key\KeyRepository;
use GuzzleHttp\Client;

/**
 * Deepgram API creator.
 */
class Deepgram {

  /**
   * The http client.
   */
  protected Client $client;

  /**
   * API Key.
   */
  private string $apiKey;

  /**
   * The base path.
   */
  private string $basePath = 'https://api.deepgram.com/v1/';

  /**
   * Constructs a new Deepgram object.
   *
   * @param \GuzzleHttp\Client $client
   *   Http client.
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The config factory.
   */
  public function __construct(Client $client, ConfigFactory $configFactory, KeyRepository $keyRepository) {
    $this->client = $client;
    $apiKey = $configFactory->get(DeepgramConfigForm::CONFIG_NAME)->get('api_key') ?? '';
    if ($apiKey) {
      $this->apiKey = $keyRepository->getKey($apiKey)->getKeyValue();
    }
  }

  /**
   * Set the api key.
   *
   * @param string $apiKey
   *   The api key.
   */
  public function setApiKey($apiKey) {
    $this->apiKey = $apiKey;
  }

  /**
   * Generate a transcription.
   *
   * @param \Drupal\file\Entity\File $file
   *   The file to transcribe.
   * @param array $config
   *   Other configs.
   *
   * @return array
   *   The transcription.
   */
  public function transcribe(File $file, $config = []) {
    if (!$this->apiKey) {
      return [];
    }
    $query['model'] = $config['model'] ?? 'general';
    if (!empty($config['language'])) {
      $query['language'] = $config['language'];
    }
    $query['detect_language'] = $this->boolToString($config['detect_language'] ?? FALSE);
    $query['punctuate'] = $this->boolToString($config['punctuate'] ?? TRUE);
    $query['diarize'] = $this->boolToString($config['diarize'] ?? FALSE);
    $query['smart_format'] = $this->boolToString($config['smart_format'] ?? FALSE);
    $query['filter_words'] = $this->boolToString($config['filter_words'] ?? FALSE);

    $guzzleOptions['headers']['Accept'] = 'application/json';
    return json_decode($this->makeRequest("listen", $query, 'POST', file_get_contents($file->getFileUri()), $guzzleOptions)->getContents(), TRUE);
  }

  /**
   * Text to speech.
   *
   * @param string $text
   *   The text.
   * @param array $config
   *   The config.
   *
   * @return string
   *   The audio binary.
   */
  public function textToSpeech($text, $config = []) {
    if (!$this->apiKey) {
      return [];
    }
    $query['model'] = $config['model'] ?? 'aura-asteria-en';
    $query['bit_rate'] = $config['bit_rate'] ?? 128000;
    $query['format'] = $config['format'] ?? 'mp3';

    $guzzleOptions['headers']['Accept'] = 'application/json';
    return $this->makeRequest("speak", $query, 'POST', $text, $guzzleOptions)->getContents();
  }

  /**
   * Transform boolean to text.
   *
   * @param bool $value
   *   The value.
   *
   * @return string
   *   The text.
   */
  protected function boolToString($value) {
    return $value ? 'true' : 'false';
  }

  /**
   * Make Deepgram call.
   *
   * @param string $path
   *   The path.
   * @param array $query_string
   *   The query string.
   * @param string $method
   *   The method.
   * @param string $body
   *   Data to attach if POST/PUT/PATCH.
   * @param array $options
   *   Extra headers.
   *
   * @return string|object
   *   The return response.
   */
  protected function makeRequest($path, array $query_string = [], $method = 'GET', $body = '', array $options = []) {
    // We can wait some.
    $options['connect_timeout'] = 30;
    $options['read_timeout'] = 30;
    // Don't let Guzzle die, just forward body and status.
    //$options['http_errors'] = FALSE;
    // Headers.
    $options['headers']['Authorization'] = 'Token ' . $this->apiKey;

    if ($body) {
      $options['body'] = $body;
    }

    $new_url = $this->basePath . $path;
    $new_url .= count($query_string) ? '?' . http_build_query($query_string) : '';

    $res = $this->client->request($method, $new_url, $options);

    return $res->getBody();
  }

}
